#include "types.h"
#include "arch.h"
#include "appconfig.h"

#define PLLCR  CLKGEN_BASE+0
#define PLLDB  CLKGEN_BASE+1
#define PLLSR  CLKGEN_BASE+2
#define IOSCTL CLKGEN_BASE+5

/* it can be specified in appconfig.h whether to wait for PLL coarse or fine state */

#ifndef OCCS_REQUIRED_LOCK_MODE
#define OCCS_REQUIRED_LOCK_MODE 0x40 /* coarse (LCK1) by default */
#endif
#if (OCCS_REQUIRED_LOCK_MODE != 0x40) && (OCCS_REQUIRED_LOCK_MODE != 0x20)
#error OCCS_REQUIRED_LOCK_MODE must be one of 0x20 (LCK0-fine) or 0x40 (LCK1-coarse)
#endif

asm void Start(void);
extern void userPreMain(void);
extern void main(void);
extern void userPostMain(void);

extern _stack_addr;
extern _bss_addr;
extern _bss_size;
extern _data_ROM_addr;
extern _data_RAM_addr;
extern _data_size;

asm void Start(void){	

/*
#ifdef COP_COPTO_INIT
	move	#COP_COPTO_INIT,X:(COP_BASE+1)
#endif

#ifdef COP_COPCTL_INIT
	move	#COP_COPCTL_INIT,X:COP_BASE
#endif

/*

	Uncomment the two instructions above if you want the COP to be
	initialized right after reset, i.e. before execution of the Start
	function. If you do so, you are then responsible for COP servicing
	during execution of the Start function or, the COP timeout must
	be set sufficiently long so that the COP timer doesn't expire before
	calling the main function, where an appropriate servicing procedure
	should always be performed.
	In case you use the COP_RESET_RECOVERY function, you should normally
	uncomment the two instructions in its definition as well.

*/
	move	#-1,x0
	move	x0,m01              /* set the m reg to linear addressing */
				
	move	hws,la              /* clear the hardware stack */
	move	hws,la
	
#ifdef BCR_INIT
    move    #BCR_INIT,x:<$FFF9          /* set waitstates in BCR*/
#endif

#ifdef SYS_CNTL_INIT
    move    #SYS_CNTL_INIT,x:SYS_BASE	/* set SYS_CNTL register */
#endif

#ifdef TARGET_MODE_0B
	bfset	#$10,x:SYS_BASE
	bfclr	#$03,OMR
#endif

	move    #_stack_addr,x0      /*init stack (definition in linker cmd file)*/
	move    x0,x:<$3F           /*set frame pointer*/
	move    x0,sp				/*set stack pointer*/
	
   
/* setup the PLL according to appconfig.h values */

#ifdef DSP56F801
#ifdef OCCS_IOSCTL_INIT
    move  #OCCS_IOSCTL_INIT,x:IOSCTL	/* adjust internal osc. frequency */
#endif
#endif

#ifdef OCCS_PLLDB_INIT
	move  #OCCS_PLLDB_INIT,x:PLLDB	/* PLLDB first, even if PLL is not used */
	nop
	nop
#endif

#ifdef OCCS_PLLCR_INIT

#ifdef DSP56F801
#if ((OCCS_PLLCR_INIT & 4) == 4) /* 801 Crystal oscillator ? */
    bfclr #$000C,x:GPIO_B_BASE   /* disable pull-ups for the EXTAL and XTAL pins */
    nop
    nop
#endif    
#endif    

#if ((OCCS_PLLCR_INIT & 3) == 2) /* PLL active ? */

#define PLLCR_TEMP (OCCS_PLLCR_INIT & 0xfc | 0x01) /* interrupts off and PLL bypassed */

	move  #$11,x0              	/* this is reset value of PLL Control Register */
	cmp   x:PLLCR,x0           	/* test PLL Control Register */
	bne   skip_pll_lock        	/* we are in simulator mode ! (skip PLL setup) */
   
	move  #PLLCR_TEMP,x:PLLCR	/* PLL lock detector ON, core still on prescaler */
	
pll_lock:
	brclr #OCCS_REQUIRED_LOCK_MODE,x:PLLSR,pll_lock	/* test lock (LCK1 or LCK0) */
	
	move  #OCCS_PLLCR_INIT,x:PLLCR	/* PLL is locked: final PLL setup */

skip_pll_lock:
	move  x:PLLSR,x0			/* clear pending clkgen interrupts */
	move  x0,x:PLLSR

#else /* ((OCCS_PLLCR_INIT & 3) == 2) PLL not active */

	move  #OCCS_PLLCR_INIT,x:PLLCR	/* just write the PLLCR init value */

#endif /* ((OCCS_PLLCR_INIT & 3) == 2) */
#endif /* OCCS_PLLCR_INIT */


#if defined (TARGET_INITDATA_XFLASH) || defined (TARGET_INITDATA_PFLASH)
    move    #_data_ROM_addr,r0  /*init global variables*/
	move    #_data_RAM_addr,r1  
	move    #_data_size,a
	tstw    a
	beq     init_vars1
init_vars:
#ifdef TARGET_INITDATA_XFLASH
	move    x:(r0)+,x0			/*from the X flash memory*/
#else
	move    p:(r0)+,x0			/*from the P flash memory*/
#endif
	decw    a	x0,x:(r1)+
	bne     init_vars
init_vars1:
#endif             /* ROM->RAM copying */


	move    #_bss_addr,r1    /*init bss variables*/
	move	#_bss_size,a
	tstw	a
	beq		init_vars_end
	move	#0,y0

init_bss_vars:
	decw    a	y0,x:(r1)+
	bne     init_bss_vars
init_vars_end:    


    /* user defined function invoked before main()*/
    /* implementation in appconfig.c*/
	jsr     userPreMain
    
    /* jump to user program after init*/
    jsr     main    
	
	/* user defined function invoked after main()*/
    /* implementation in appconfig.c*/
	jmp     userPostMain

}
