/******************************************************************************
*
* Motorola Inc.
* (c) Copyright 2002 Motorola, Inc.
* ALL RIGHTS RESERVED.
*
*******************************************************************************
*
* FILE NAME: gpio.h
*
* DESCRIPTION: Header file for GPIO driver
*
* MODULES INCLUDED: none
*
* NOTES:
*
*******************************************************************************/

#ifndef __GPIO_H
#define __GPIO_H

#if !defined(__ARCH_H) || !defined(__PERIPH_H)
#error arch.h and periph.h must be included before gpio.h
#endif

#ifdef __cplusplus
extern "C" {
#endif

/************************************************************
* GPIO module identifiers  (see also gpiotarget.h)
*************************************************************/

#define GPIO_A	(&ArchIO.PortA)
#define GPIO_B	(&ArchIO.PortB)
#define GPIO_D	(&ArchIO.PortD)
#define GPIO_E	(&ArchIO.PortE)


/******************************************************************************
*
*                      General Interface Description
*
* The DSP56F801/803/805/807 general purpose input/output (GPIO) is designed to share
* package pins with other peripherals on the chip. If a peripheral is not required,
* the pin may be programmed as input, output, or level-sensitive interrupt input.
* GPIOs are placed on the chip in groups of eight bits. For information about which
* GPIO pins are multiplexed and shared with other peripherals, please refer to
* User's Manual's Chapter Two.
* 
* - DSP56F801 - Has no dedicated GPIOs; but 11 multiplexed GPIOs on Ports A and B
* - DSP56F803 - Has no dedicated GPIOs; but 16 multiplexed GPIOs on Ports A and E
* - DSP56F805 - Has eight dedicated pins on Port B while the lower six pins of
*               Port D are dedicated GPIO. There are 18 multiplexed GPIOs available
*               on Ports A, D, and E
* - DSP56F807 - Has 14 dedicated pins on Ports B and D while there are 18 multiplexed
*               GPIOs available on Ports A, D, and E
* 
* For more information, please refer to the DSP56F80x User's Manual,
* for a description of the GPIO device.
*
* The GPIO device driver interface uses "ioctl" call to control specific
* GPIO functions during operation. For details see "ioctl" call.
*      
*
******************************************************************************/

/****************************************************
* Configurable items, i.e. defines for appconfig.h
*****************************************************

  void gpioxISR( void );         prototype for user ISR 
  #define INT_VECTOR_ADDR_yy     gpioxISR
  #define INT_PRIORITY_LEVEL_yy   value 0-7 (0 = interrupt disabled)

  #define  GPIO_A_PUR_INIT      0x0000
  #define  GPIO_A_DDR_INIT      0x0000
  #define  GPIO_A_PER_INIT      0x0000
  #define  GPIO_A_IENR_INIT     0x0000
  #define  GPIO_A_IPOLR_INIT    0x0000

  dtto for B, D, E
  
  where:
         yy  is interrupt vector number

                23 - GPIO A
                22 - GPIO B
                20 - GPIO D
                19 - GPIO E
*/

/*****************************************************************
* ioctl() commands
*
*  Word16 ioctl( PWM_module_identifier, Command_name, Parameter );
*
******************************************************************

 COMMAND NAME                      PARAMETERS, COMMENTS
--------------------------------------------------------*/

#define GPIO_INIT                             /* NULL */ 
#define GPIO_SETAS_GPIO                       /* BIT_0 | BIT_1 | BIT_2 | BIT_3 | BIT_4 | BIT_5 | BIT_6 | BIT_7 */
#define GPIO_SETAS_PERIPHERAL                 /* BIT_0 | BIT_1 | BIT_2 | BIT_3 | BIT_4 | BIT_5 | BIT_6 | BIT_7 */
#define GPIO_SETAS_INPUT                      /* BIT_0 | BIT_1 | BIT_2 | BIT_3 | BIT_4 | BIT_5 | BIT_6 | BIT_7 */
#define GPIO_SETAS_OUTPUT                     /* BIT_0 | BIT_1 | BIT_2 | BIT_3 | BIT_4 | BIT_5 | BIT_6 | BIT_7 */
#define GPIO_INT_DISABLE                      /* BIT_0 | BIT_1 | BIT_2 | BIT_3 | BIT_4 | BIT_5 | BIT_6 | BIT_7 */
#define GPIO_INT_ENABLE                       /* BIT_0 | BIT_1 | BIT_2 | BIT_3 | BIT_4 | BIT_5 | BIT_6 | BIT_7 */
#define GPIO_PULLUP_DISABLE                   /* BIT_0 | BIT_1 | BIT_2 | BIT_3 | BIT_4 | BIT_5 | BIT_6 | BIT_7 */
#define GPIO_PULLUP_ENABLE                    /* BIT_0 | BIT_1 | BIT_2 | BIT_3 | BIT_4 | BIT_5 | BIT_6 | BIT_7 */

#define GPIO_SW_INT_ASSERT						/* BIT_0 | BIT_1 | BIT_2 | BIT_3 | BIT_4 | BIT_5 | BIT_6 | BIT_7 */
//#define GPIO_INT_ASSERT_ENABLE                /* BIT_0 | BIT_1 | BIT_2 | BIT_3 | BIT_4 | BIT_5 | BIT_6 | BIT_7 */
#define GPIO_CLEAR_SW_INT_PENDING				/* BIT_0 | BIT_1 | BIT_2 | BIT_3 | BIT_4 | BIT_5 | BIT_6 | BIT_7 */
//#define GPIO_INT_ASSERT_DISABLE               /* BIT_0 | BIT_1 | BIT_2 | BIT_3 | BIT_4 | BIT_5 | BIT_6 | BIT_7 */

#define GPIO_INT_DETECTION_ACTIVE_HIGH        /* BIT_0 | BIT_1 | BIT_2 | BIT_3 | BIT_4 | BIT_5 | BIT_6 | BIT_7 */
#define GPIO_INT_DETECTION_ACTIVE_LOW         /* BIT_0 | BIT_1 | BIT_2 | BIT_3 | BIT_4 | BIT_5 | BIT_6 | BIT_7 */
#define GPIO_SET_PIN                          /* BIT_0 | BIT_1 | BIT_2 | BIT_3 | BIT_4 | BIT_5 | BIT_6 | BIT_7 */
#define GPIO_CLEAR_PIN                        /* BIT_0 | BIT_1 | BIT_2 | BIT_3 | BIT_4 | BIT_5 | BIT_6 | BIT_7 */
#define GPIO_TOGGLE_PIN                       /* BIT_0 | BIT_1 | BIT_2 | BIT_3 | BIT_4 | BIT_5 | BIT_6 | BIT_7 */
#define GPIO_READ_DATA                        /* NULL */
#define GPIO_WRITE_DATA                       /* BIT_0 | BIT_1 | BIT_2 | BIT_3 | BIT_4 | BIT_5 | BIT_6 | BIT_7 */
#define GPIO_READ_INT_PENDING_REG             /* NULL */
#define GPIO_GET_INT_PENDING_FLAG             /* BIT_0 | BIT_1 | BIT_2 | BIT_3 | BIT_4 | BIT_5 | BIT_6 | BIT_7 */
#define GPIO_TEST_INT_PENDING	              /* BIT_0 | BIT_1 | BIT_2 | BIT_3 | BIT_4 | BIT_5 | BIT_6 | BIT_7 */

#define GPIO_CLEAR_INT_PENDING				  /* value read from INT_PENDING_REG */
//#define GPIO_CLEAR_EDGE_SENSITIVE_INT_FLAG    /* value read from INT_PENDING_REG */

/****************************************************************************
* GPIO constants used in the ioctl functions
*****************************************************************************/
#define BIT_0	0x0001
#define BIT_1	0x0002
#define BIT_2	0x0004
#define BIT_3	0x0008
#define BIT_4	0x0010
#define BIT_5	0x0020
#define BIT_6	0x0040
#define BIT_7	0x0080

/*******************************************************************************
  Implementations of GPIO ioctl() commands
*******************************************************************************/

/* module names for internal use in ioctl implementation */

#define __GPIO_A GPIO_A
#define __GPIO_B GPIO_B
#define __GPIO_D GPIO_D
#define __GPIO_E GPIO_E

/* GPIO Init */

#define ioctlGPIO_INIT(pGpioBase, param) gpioInit(pGpioBase)
void gpioInit(arch_sPort *pGpioBase);  /* function prototype - implemented in gpio.c */

/* GPIO Peripheral Enable Register */

#define ioctlGPIO_SETAS_GPIO(pGpioBase, param)		\
  periphBitClear (param,&pGpioBase->per)
						
#define ioctlGPIO_SETAS_PERIPHERAL(pGpioBase, param)     \
  periphBitSet   (param,&pGpioBase->per)

/* GPIO Data Direction Register */

#define ioctlGPIO_SETAS_INPUT(pGpioBase, param)	\
  periphBitClear   (param,&pGpioBase->ddr)

#define ioctlGPIO_SETAS_OUTPUT(pGpioBase, param)	\
  periphBitSet   (param,&pGpioBase->ddr)

/* GPIO Data Register */

#define ioctlGPIO_SET_PIN(pGpioBase, param)				\
  periphBitSet(param,&pGpioBase->dr)

#define ioctlGPIO_CLEAR_PIN(pGpioBase, param)			\
  periphBitClear (param,&pGpioBase->dr)

#define ioctlGPIO_TOGGLE_PIN(pGpioBase, param)			\
  periphBitChange (param,&pGpioBase->dr)

#define ioctlGPIO_READ_DATA(pGpioBase, param)     \
  periphMemRead(&pGpioBase->dr)

#define ioctlGPIO_WRITE_DATA(pGpioBase, param)     \
  periphMemWrite(param,&pGpioBase->dr)

/* GPIO Pull-Up Enable Register */

#define ioctlGPIO_PULLUP_DISABLE(pGpioBase, param)			\
  periphBitClear (param,&pGpioBase->pur)

#define ioctlGPIO_PULLUP_ENABLE(pGpioBase, param)        \
  periphBitSet   (param,&pGpioBase->pur)

/* GPIO Interrupt Assert Register */


#define ioctlGPIO_CLEAR_SW_INT_PENDING(pGpioBase, param)     \
  periphBitClear(param,&pGpioBase->iar)

#define ioctlGPIO_SW_INT_ASSERT(pGpioBase, param)     \
  periphBitSet   (param,&pGpioBase->iar)


#define ioctlGPIO_INT_ASSERT_DISABLE(pGpioBase, param)     \
  periphBitClear(param,&pGpioBase->iar)

#define ioctlGPIO_INT_ASSERT_ENABLE(pGpioBase, param)     \
  periphBitSet   (param,&pGpioBase->iar)

/* GPIO Interrupt Enable Register */

#define ioctlGPIO_INT_DISABLE(pGpioBase, param)    \
  periphBitClear(param,&pGpioBase->ienr)

#define ioctlGPIO_INT_ENABLE(pGpioBase, param)     \
  periphBitSet   (param,&pGpioBase->ienr)

/* GPIO Interrupt Polarity Register */

#define ioctlGPIO_INT_DETECTION_ACTIVE_HIGH(pGpioBase, param)    \
  periphBitClear(param,&pGpioBase->ipolr)

#define ioctlGPIO_INT_DETECTION_ACTIVE_LOW(pGpioBase, param)     \
  periphBitSet   (param,&pGpioBase->ipolr)

/* GPIO Interrupt Edge Sensitive Register */

#define ioctlGPIO_CLEAR_INT_PENDING(pGpioBase, param)     \
  periphBitClear(param,&pGpioBase->iesr)

#define ioctlGPIO_CLEAR_EDGE_SENSITIVE_INT_FLAG(pGpioBase, param)     \
  periphBitClear(param,&pGpioBase->iesr)

/* GPIO Interrupt Pending Register */

#define ioctlGPIO_READ_INT_PENDING_REG(pGpioBase, param)     \
  periphMemRead(&pGpioBase->ipr)
  
#define ioctlGPIO_GET_INT_PENDING_FLAG(pGpioBase, param)     \
  ((param) & periphMemRead(&pGpioBase->ipr))

#define ioctlGPIO_TEST_INT_PENDING(pGpioBase, param)     \
  !((param)  & ~periphMemRead(&pGpioBase->ipr))


/**********************************************************************
* Include of device specific GPIO commands and defines
***********************************************************************/
#include "gpiotarget.h"  /* device specific defines */

#ifdef __cplusplus
}
#endif

#endif										

