/*****************************************************************************
*
* Motorola Inc.
* (c) Copyright 2002 Motorola, Inc.
* ALL RIGHTS RESERVED.
*
******************************************************************************
*
* File Name: mscan.c
*
* Description:  msCAN (Motorola Scallable CAN) driver
*
* Modules Included:
*
*****************************************************************************/

#include "types.h"
#include "arch.h"
#include "periph.h"

#include "appconfig.h"
#include "mscan.h"

#ifdef __cplusplus
extern "C" {
#endif

/* initialization of msCAN */

void mscan_Init(arch_sMSCAN *pMSCanBase)
{
	// enter soft-reset state
	ioctl(pMSCanBase, MSCAN_SOFT_RESET, MSCAN_ON);

	// initialize control registers writable in reset-state only
	#ifdef MSCAN_CANIDAC_INIT
	periphMemWrite(MSCAN_CANIDAC_INIT,&pMSCanBase->canidac);
	#endif

	#ifdef MSCAN_CANIDAR0_INIT
	periphMemWrite(MSCAN_CANIDAR0_INIT,&pMSCanBase->canidar0);
	#endif
	#ifdef MSCAN_CANIDAR1_INIT
	periphMemWrite(MSCAN_CANIDAR1_INIT,&pMSCanBase->canidar1);
	#endif
	#ifdef MSCAN_CANIDAR2_INIT
	periphMemWrite(MSCAN_CANIDAR2_INIT,&pMSCanBase->canidar2);
	#endif
	#ifdef MSCAN_CANIDAR3_INIT
	periphMemWrite(MSCAN_CANIDAR3_INIT,&pMSCanBase->canidar3);
	#endif
	#ifdef MSCAN_CANIDAR4_INIT
	periphMemWrite(MSCAN_CANIDAR4_INIT,&pMSCanBase->canidar4);
	#endif
	#ifdef MSCAN_CANIDAR5_INIT
	periphMemWrite(MSCAN_CANIDAR5_INIT,&pMSCanBase->canidar5);
	#endif
	#ifdef MSCAN_CANIDAR6_INIT
	periphMemWrite(MSCAN_CANIDAR6_INIT,&pMSCanBase->canidar6);
	#endif
	#ifdef MSCAN_CANIDAR7_INIT
	periphMemWrite(MSCAN_CANIDAR7_INIT,&pMSCanBase->canidar7);
	#endif
	
	#ifdef MSCAN_CANIDMR0_INIT
	periphMemWrite(MSCAN_CANIDMR0_INIT,&pMSCanBase->canidmr0);
	#endif
	#ifdef MSCAN_CANIDMR1_INIT
	periphMemWrite(MSCAN_CANIDMR1_INIT,&pMSCanBase->canidmr1);
	#endif
	#ifdef MSCAN_CANIDMR2_INIT
	periphMemWrite(MSCAN_CANIDMR2_INIT,&pMSCanBase->canidmr2);
	#endif
	#ifdef MSCAN_CANIDMR3_INIT
	periphMemWrite(MSCAN_CANIDMR3_INIT,&pMSCanBase->canidmr3);
	#endif
	#ifdef MSCAN_CANIDMR4_INIT
	periphMemWrite(MSCAN_CANIDMR4_INIT,&pMSCanBase->canidmr4);
	#endif
	#ifdef MSCAN_CANIDMR5_INIT
	periphMemWrite(MSCAN_CANIDMR5_INIT,&pMSCanBase->canidmr5);
	#endif
	#ifdef MSCAN_CANIDMR6_INIT
	periphMemWrite(MSCAN_CANIDMR6_INIT,&pMSCanBase->canidmr6);
	#endif
	#ifdef MSCAN_CANIDMR7_INIT
	periphMemWrite(MSCAN_CANIDMR7_INIT,&pMSCanBase->canidmr7);
	#endif

	#ifdef MSCAN_CANBTR0_INIT
	periphMemWrite(MSCAN_CANBTR0_INIT,&pMSCanBase->canbtr0);
	#endif
	
	#ifdef MSCAN_CANBTR1_INIT
	periphMemWrite(MSCAN_CANBTR1_INIT,&pMSCanBase->canbtr1);
	#endif

	#ifdef MSCAN_CANCTL1_INIT
	periphMemWrite(MSCAN_CANCTL1_INIT,&pMSCanBase->canctl1);
	#endif
	
	// leave soft-reset state
	ioctl(pMSCanBase, MSCAN_SOFT_RESET, MSCAN_OFF);

	#ifdef MSCAN_CANRIER_INIT
	periphMemWrite(MSCAN_CANRIER_INIT,&pMSCanBase->canrier);
	#endif

	#ifdef MSCAN_CANTCR_INIT
	periphMemWrite(MSCAN_CANTCR_INIT,&pMSCanBase->cantcr);
	#endif
	
	// initialize the rest of main control register
	#ifdef MSCAN_CANCTL0_INIT
	periphMemWrite(MSCAN_CANCTL0_INIT,&pMSCanBase->canctl0);
	#endif
}

// ID->RAW calculation implemented as inline assembly

#pragma interrupt called
asm UWord32 MSCAN_Id2Idr_V(register UWord32 id)
{		// id is in A register
	lea		(SP)+
	move	X0,X:(SP)+
	move	Y0,X:(SP)
	
	move	A0,Y0
	brclr	#0x8000, A1, std_id

ext_id:
	lsl		Y0
	rol		A
	move	A1,X0
	lsl		A
	lsl		A
	bfset	#0xFFF8,X0
	bfset	#0x001F,A1
	and		X0,A
	move	Y0,A0
	bra rtrn

std_id:
	move	#5,X0
	asll	Y0,X0,A

rtrn:

	move	X:(SP)-,Y0
	move	X:(SP)-,X0
	rts
}

// RAW->ID calculation implemented as inline assembly

#pragma interrupt called
asm UWord32 MSCAN_Idr2Id_V(register UWord32 idr)
{
	lea		(SP)+
	move	X0,X:(SP)+
	move	Y0,X:(SP)

	brclr	#8,A1,std_id
	
ext_id:
	move	A0,Y0
	lsr		A
	ror		Y0
	move	A1,X0
	lsr		A
	lsr		A
	bfclr	#0xFFFC,X0
	bfclr	#0x0003,A1
	or		X0,A
	bfset	#0x8000,A1
	bra		rtrn
	
std_id:
	move	#5,Y0
	lsrr	A1,Y0,Y0
	clr		A1

rtrn:
	move	Y0,A0

	move	X:(SP)-,Y0
	move	X:(SP)-,X0
	rts
}


/* same as MSCANMB_SET_ID but implemented as function (to be used when ID passed is a variable) */

#pragma interrupt called
void ioctlMSCANMB_SET_ID_V(register arch_sMSCAN_MB* pMSCanMB, register UWord32 id)
{
	register UWord32 idr = MSCAN_Id2Idr_V(id);
	
	// set RTR ?
	if(id & MSCAN_ID_RTR)
		idr |= MSCAN_MB_IDR3_ERTR | (((UWord32)MSCAN_MB_IDR1_SRTR) << 16);
				
	ioctl(pMSCanMB, MSCANMB_SET_ID_RAW, idr);
}

/* Retrieve 32bit ID from the apropriate bits in given MB */

#pragma interrupt called
asm UWord32 ioctlMSCANMB_GET_ID(register arch_sMSCAN_MB* pMSCanMB, void* param)
{
	lea		(SP)+
	move	X0,X:(SP)+
	move	Y0,X:(SP)

	move	#8,X0
	move	X:(R2+0),Y0
	asll	Y0,X0,A
	move	X:(R2+1),Y0
	or		Y0,A
	brclr	#8,A1,std_id
	
ext_id:
	move	X:(R2+2),Y0
	asll	Y0,X0,X0
	move	X:(R2+3),Y0
	or		X0,Y0

	lsr		A
	ror		Y0
	move	A1,X0
	lsr		A
	lsr		A
	bfclr	#0xFFFC,X0
	bfclr	#0x0003,A1
	or		X0,A
	bfset	#0x8000,A1
	bra		rtrn
	
std_id:
	move	#5,Y0
	lsrr	A1,Y0,Y0
	clr		A1

rtrn:
	move	Y0,A0

	move	X:(SP)-,Y0
	move	X:(SP)-,X0
	rts
}

#ifdef __cplusplus
}
#endif

