/******************************************************************************
*
* Motorola Inc.
* (c) Copyright 2002 Motorola, Inc.
* ALL RIGHTS RESERVED.
*
*******************************************************************************
*
* FILE NAME: pwm.c   
*
* DESCRIPTION: Source file for PWM Driver
*
* MODULES INCLUDED: types.h
*                   arch.h
*                   periph.h
*                   appconfig.h
*                   mc.h
*                   pwm.h
*
* NOTES:
*
*******************************************************************************/

#include "types.h"
#include "arch.h"
#include "periph.h"

#include "appconfig.h"
#include "mc.h"
#include "pwm.h"

#ifdef __cplusplus
extern "C" {
#endif


/********************************************************************************
 pwmInit() function performs the PWM module static configuration
 based on the configurable items from appconfig.h 
********************************************************************************/
void pwmInit(arch_sPWM *pPwmBase)
{
#ifdef PWM_A
	if(pPwmBase == PWM_A)
	{  
		#ifdef PWM_A_PMFCTL_INIT
		periphMemWrite(PWM_A_PMFCTL_INIT, &pPwmBase->pmfctl);
		#endif
		#ifdef PWM_A_PMFSA_INIT
		periphMemWrite(PWM_A_PMFSA_INIT, &pPwmBase->pmfsa);
		#endif
		#ifdef PWM_A_PWMCM_INIT
		periphMemWrite(PWM_A_PWMCM_INIT, &pPwmBase->pwmcm);
		#endif
		#ifdef PWM_A_PMDEADTM_INIT
		periphMemWrite(PWM_A_PMDEADTM_INIT, &pPwmBase->pmdeadtm);
		#endif
		#ifdef PWM_A_PMDISMAP1_INIT
		periphMemWrite(PWM_A_PMDISMAP1_INIT, &pPwmBase->pmdismap1);
		#endif
		#ifdef PWM_A_PMDISMAP2_INIT
		periphMemWrite(PWM_A_PMDISMAP2_INIT, &pPwmBase->pmdismap2);
		#endif
		#ifdef PWM_A_PMCCR_INIT
        if((PWM_A_PMCCR_INIT & PWM_PMCCR_ENHA) > 0 )  /* Hardware Acceleration enabled inside GCT */
			periphBitSet(PWM_PMCCR_ENHA, &pPwmBase->pmccr);
		periphMemWrite(PWM_A_PMCCR_INIT, &pPwmBase->pmccr);
		#endif
		#ifdef PWM_A_PMCFG_INIT
		periphMemWrite(PWM_A_PMCFG_INIT, &pPwmBase->pmcfg);
		#endif
		#ifdef PWM_A_PMOUT_INIT
		periphMemWrite(PWM_A_PMOUT_INIT, &pPwmBase->pmout);
		#endif
		#ifdef PWM_A_PMCTL_INIT
		periphMemWrite(PWM_A_PMCTL_INIT, &pPwmBase->pmctl);
		if((PWM_A_PMCTL_INIT & PWM_PMCTL_LDOK) > 0 )  /* Load OK set inside GCT */
			periphBitSet(PWM_PMCTL_LDOK, &pPwmBase->pmctl);
		#endif

	}
	else
#endif
#ifdef PWM_B
	if(pPwmBase == PWM_B)
	{  
		#ifdef PWM_B_PMFCTL_INIT
		periphMemWrite(PWM_B_PMFCTL_INIT, &pPwmBase->pmfctl);
		#endif
		#ifdef PWM_B_PMFSA_INIT
		periphMemWrite(PWM_B_PMFSA_INIT, &pPwmBase->pmfsa);
		#endif
		#ifdef PWM_B_PWMCM_INIT
		periphMemWrite(PWM_B_PWMCM_INIT, &pPwmBase->pwmcm);
		#endif
		#ifdef PWM_B_PMDEADTM_INIT
		periphMemWrite(PWM_B_PMDEADTM_INIT, &pPwmBase->pmdeadtm);
		#endif
		#ifdef PWM_B_PMDISMAP1_INIT
		periphMemWrite(PWM_B_PMDISMAP1_INIT, &pPwmBase->pmdismap1);
		#endif
		#ifdef PWM_B_PMDISMAP2_INIT
		periphMemWrite(PWM_B_PMDISMAP2_INIT, &pPwmBase->pmdismap2);
		#endif
		#ifdef PWM_B_PMCCR_INIT
		if((PWM_B_PMCCR_INIT & PWM_PMCCR_ENHA) > 0 )  /* Hardware Acceleration enabled inside GCT */
			periphBitSet(PWM_PMCCR_ENHA, &pPwmBase->pmccr);
		periphMemWrite(PWM_B_PMCCR_INIT, &pPwmBase->pmccr);
		#endif
		#ifdef PWM_B_PMCFG_INIT
		periphMemWrite(PWM_B_PMCFG_INIT, &pPwmBase->pmcfg);
		#endif
		#ifdef PWM_B_PMOUT_INIT
		periphMemWrite(PWM_B_PMOUT_INIT, &pPwmBase->pmout);
		#endif
		#ifdef PWM_B_PMCTL_INIT
		periphMemWrite(PWM_B_PMCTL_INIT, &pPwmBase->pmctl);
		if((PWM_B_PMCTL_INIT & PWM_PMCTL_LDOK) > 0 )  /* Load OK set inside GCT */
			periphBitSet(PWM_PMCTL_LDOK, &pPwmBase->pmctl);
		#endif
	}
	else
#endif

	; /* last else case */
}

#ifdef __cplusplus
}
#endif
