/*****************************************************************************
*
* Motorola Inc.
* (c) Copyright 2002 Motorola, Inc.
* ALL RIGHTS RESERVED.
*
******************************************************************************
*
* File Name: arch.h
*
* Description: Architecture Dependent Declarations for the DSP56F805 device
*
* Modules Included:
*
*****************************************************************************/
#ifndef __ARCH_H
#define __ARCH_H

typedef volatile union{
	struct { 
	 UWord16 ControlReg;
	 UWord16 StatusReg; 
	 UWord16 Reserved[14];
	};
	struct { 
	 UWord16 cntl;
	 UWord16 sts;
	 UWord16 reserved[14];
	};
} arch_sSIM;

typedef volatile union{
	struct {
	UWord16 CompareReg1;
	UWord16 CompareReg2;
	UWord16 CaptureReg;
	UWord16 LoadReg;
	UWord16 HoldReg;
	UWord16 CounterReg;
	UWord16 ControlReg;
	UWord16 StatusControlReg;
	};
	struct {
	 UWord16 cmp1; 
	 UWord16 cmp2;
	 UWord16 cap;
	 UWord16 load;
	 UWord16 hold;
	 UWord16 cntr;
	 UWord16 ctrl;
	 UWord16 scr;
	};
} arch_sTimerChannel;

typedef volatile union{
	struct {
	arch_sTimerChannel Channel0;
	arch_sTimerChannel Channel1;
	arch_sTimerChannel Channel2;
	arch_sTimerChannel Channel3;
	};
	struct {
 	 arch_sTimerChannel ch0; 
	 arch_sTimerChannel ch1; 
	 arch_sTimerChannel ch2; 
	 arch_sTimerChannel ch3; 
	};
} arch_sTimer;

typedef volatile union{
	struct{
	UWord16 IdReg0;
	UWord16 IdReg1;
	UWord16 IdReg2;
	UWord16 IdReg3;
	UWord16 DataSegReg[8];
	UWord16 DataLengthReg;

	union{
		UWord16 TxBufferPriorityReg;
		UWord16 Reserved1;
	} U;

	UWord16 Reserved2[2];
    };
    
	struct{
	 union {
		UWord16 idr[4];
		struct {
	 	 UWord16 idr0;
	 	 UWord16 idr1;
	 	 UWord16 idr2;
	 	 UWord16 idr3;
		};
	  };
	 union {
		UWord16 dsr[8];
		struct{
		 UWord16 dsr0;
		 UWord16 dsr1;
		 UWord16 dsr2;
		 UWord16 dsr3;
		 UWord16 dsr4;
		 UWord16 dsr5;
		 UWord16 dsr6;
		 UWord16 dsr7;
		}; 
	 };
   	 UWord16 dlr;
	 UWord16 tbpr;

	 UWord16 reserved2[2];
	};
} arch_sMSCAN_MB;

typedef volatile union{
	struct {
	 UWord16 Control0Reg;
	 UWord16 Control1Reg;
	 UWord16 BusTiming0Reg;
	 UWord16 BusTiming1Reg;
	 UWord16 RxFlagReg;
	 UWord16 RxIntEnableReg;
	 UWord16 TxFlagReg;
	 UWord16 TxControlReg;
	 UWord16 IdControlReg;

	 UWord16 Reserved1[5];
	
	 UWord16 RxErrorCountReg;
	 UWord16 TxErrorCountReg;
	 UWord16 IdAcceptReg0;
	 UWord16 IdAcceptReg1;
	 UWord16 IdAcceptReg2;
	 UWord16 IdAcceptReg3;
	 UWord16 IdMaskReg0;
	 UWord16 IdMaskReg1;
	 UWord16 IdMaskReg2;
	 UWord16 IdMaskReg3;
	 UWord16 IdAcceptReg4;
	 UWord16 IdAcceptReg5;
	 UWord16 IdAcceptReg6;
	 UWord16 IdAcceptReg7;
	 UWord16 IdMaskReg4;
	 UWord16 IdMaskReg5;
	 UWord16 IdMaskReg6;
	 UWord16 IdMaskReg7;

	 UWord16 Reserved2[0x20];
	 
	arch_sMSCAN_MB RxBuffer;
	arch_sMSCAN_MB TxBuffer[3];
	};
	struct {
	 UWord16 canctl0; 
	 UWord16 canctl1; 
	 UWord16 canbtr0; 
	 UWord16 canbtr1; 
	 UWord16 canrflg; 
	 UWord16 canrier; 
	 UWord16 cantflg; 
	 UWord16 cantcr; 
	 UWord16 canidac; 

	 UWord16 reserved1[5];
	
	 UWord16 canrxerr; 
	 UWord16 cantxerr; 
	 UWord16 canidar0; 
	 UWord16 canidar1; 
	 UWord16 canidar2; 
	 UWord16 canidar3; 
	 UWord16 canidmr0; 
	 UWord16 canidmr1; 
	 UWord16 canidmr2; 
	 UWord16 canidmr3; 
	 UWord16 canidar4; 
	 UWord16 canidar5; 
	 UWord16 canidar6; 
	 UWord16 canidar7; 
	 UWord16 canidmr4; 
	 UWord16 canidmr5; 
	 UWord16 canidmr6; 
	 UWord16 canidmr7; 

	 UWord16 reserved2[0x20];
	 
	 union {	
		struct {
		 arch_sMSCAN_MB canrb;
		 arch_sMSCAN_MB cantb[3];
		};
		arch_sMSCAN_MB canmb[4];
	 };
	};
} arch_sMSCAN;

typedef volatile union{
	struct{
	UWord16 ControlReg;
	UWord16 FaultControlReg;
	UWord16 FaultStatusReg;
	UWord16 OutputControlReg;
	UWord16 CounterReg;
	UWord16 CounterModuloReg;
	 Word16  ValueReg[6];
	UWord16 DeadtimeReg;
	UWord16 DisableMapping1Reg;
	UWord16 DisableMapping2Reg;
	UWord16 ConfigReg;
	UWord16 ChannelControlReg;
	UWord16 PortReg;
			
	 UWord16 Reserved[14];
	};
	struct{
	 UWord16 pmctl; 
	 UWord16 pmfctl; 
	 UWord16 pmfsa; 
	 UWord16 pmout; 
	 UWord16 pmcnt; 
	 UWord16 pwmcm; 
	 Word16  pwmval[6];
	 UWord16 pmdeadtm; 
	 UWord16 pmdismap1; 
	 UWord16 pmdismap2; 
	 UWord16 pmcfg; 
	 UWord16 pmccr; 
	 UWord16 pmport; 
			
	 UWord16 reserved[14];
	};
} arch_sPWM;

typedef volatile union{
    struct{
	UWord16 ControlReg;
	UWord16 FilterIntervalReg;
	UWord16 WatchdogTimeoutReg;
	Word16  PositionDifferenceReg; 
	Word16  PositionDifferenceHoldReg;
	Word16  RevolutionCounterReg; 
	Word16  RevolutionHoldReg;  
	Word16  UpperPositionCounterReg;
	UWord16 LowerPositionCounterReg;
	Word16  UpperPositionHoldReg;  
	UWord16 LowerPositionHoldReg;
	Word16  UpperInitializationReg;  
	UWord16 LowerInitializationReg;
	UWord16 InputMonitorReg;
	UWord16 TestReg;
	UWord16 Reserved;
    };
    struct{
	 UWord16 deccr; 
	 UWord16 fir; 
	 UWord16 wtr; 
	 Word16  posd; 
	 Word16  posdh; 
	 Word16  rev; 
	 Word16  revh; 
	 Word16  upos; 
	 UWord16 lpos; 
	 Word16  uposh; 
	 UWord16 lposh; 
	 Word16  uir; 
	 Word16  lir; 
	 UWord16 imr; 
	 UWord16 tstreg; 
			
	 UWord16 reserved;
    };
} arch_sDecoder;

typedef volatile union{
	struct{
	UWord16 GroupPriorityReg[16];
	UWord16 IntRequestReg[4];
	UWord16 Reserved1[4];
	UWord16 IntSourceReg[4];
	UWord16 ControlReg;
	UWord16 Reserved2[3];
	};

	struct{
	 UWord16 gpr[16];	
	 UWord16 tirq[4];
	 UWord16 reserved1[4];
	 UWord16 tisr[4];
	 UWord16 tcsr;	
	 UWord16 reserved2[3];
	};
	
} arch_sIntCntrl;

typedef volatile union{
	struct{
	UWord16 Control1Reg;
	UWord16 Control2Reg;
	UWord16 ZeroCrossControlReg;
	UWord16 ChannelList1Reg;
	UWord16 ChannelList2Reg;
	UWord16 DisableReg;
	UWord16 StatusReg;
	UWord16 LimitReg;
	UWord16 ZeroCrossStatusReg;
	UWord16 ResultReg[8];
	UWord16 LowLimitReg[8];
	UWord16 HighLimitReg[8];
	UWord16 OffsetReg[8];
	UWord16 Reserved[23];
	};
	struct{
	 UWord16 adcr1;
	 UWord16 adcr2;
	 UWord16 adzcc;
	 UWord16 adlst1;
	 UWord16 adlst2;
	 UWord16 adsdis;
	 UWord16 adstat;
	 UWord16 adlstat;
	 UWord16 adzcstat;
	 UWord16 adrslt[8];
	 UWord16 adllmt[8];
	 UWord16 adhlmt[8];
	 UWord16 adofs[8];

	 UWord16 reserved[23];
	};
} arch_sADC;

typedef volatile union{
	struct{
	UWord16 BaudRateReg;
	UWord16 ControlReg;
	UWord16 StatusReg;
	UWord16 DataReg;
	
	 UWord16 Reserved[12];	
	};
	struct{
	 UWord16 scibr;
	 UWord16 scicr;
	 UWord16 scisr;
	 UWord16 scidr;
	
	 UWord16 reserved[12];	
	};
} arch_sSCI;

typedef volatile union{
	struct{
	UWord16 ControlReg;
	UWord16 DataSizeReg;
	UWord16 DataRxReg;
	UWord16 DataTxReg;
	
	 UWord16 Reserved[12];
	};
	struct{
	 UWord16 spscr;
	 UWord16 spdsr;
	 UWord16 spdrr;
	 UWord16 spdtr;
	
	 UWord16 reserved[12];
	};
} arch_sSPI;

typedef volatile union{
	struct{
	UWord16 ControlReg;
	UWord16 TimeoutReg;
	UWord16 ServiceReg;
	UWord16 Reserved[13];
	};
	struct{
	 UWord16 copctl;
	 UWord16 copto;
	 UWord16 copsrv;
			
	 UWord16 reserved[13];
	};
} arch_sCOP;

typedef volatile union{
	struct{
	UWord16 ControlReg;
	UWord16 ProgramReg;
	UWord16 EraseReg;
	UWord16 AddressReg;
	UWord16 DataReg;
	UWord16 IntReg;
	UWord16 IntSourceReg;
	UWord16 IntPendingReg;
	UWord16 DivisorReg;
	UWord16 TimerEraseReg;
	UWord16 TimerMassEraseReg;
	UWord16 TimerNVStorageReg;
	UWord16 TimerProgramSetupReg;
	UWord16 TimerProgramReg;
	UWord16 TimerNVHoldReg;
	UWord16 TimerNVHold1Reg;
	UWord16 TimerRecoveryReg;
	UWord16 Reserved[15];
	};
	struct{
	 UWord16 cntl;
	 UWord16 pe;
	 UWord16 ee;
	 UWord16 addr;
	 UWord16 data;
	 UWord16 ie;
	 UWord16 is;
	 UWord16 ip;
	 UWord16 ckdivisor;
	 UWord16 terasel;
	 UWord16 tmel;
	 UWord16 tnvsl;
	 UWord16 tpgsl;
	 UWord16 tprogl;
	 UWord16 tnvhl;
	 UWord16 tnvh1l;
	 UWord16 trcvl;
			
	 UWord16 reserved[15];
	};
} arch_sFlash;

typedef volatile union{
	struct{
	 UWord16 ControlReg;
	 UWord16 DivideReg;
	 UWord16 StatusReg;
	 UWord16 TestReg;
	 UWord16 SelectReg;
	
	 UWord16 Reserved[11];
	}; 
	struct{
	 UWord16 pllcr;
	 UWord16 plldb;
	 UWord16 pllsr;
	 UWord16 testr;
	 UWord16 clkosr;
	
	 UWord16 reserved[11];
	}; 
} arch_sPLL;

typedef volatile union{
	struct{
	UWord16 PullUpReg;
	UWord16 DataReg;
	UWord16 DataDirectionReg;
	UWord16 PeripheralReg;
	UWord16 IntAssertReg;
	UWord16 IntEnableReg;
	UWord16 IntPolarityReg;
	UWord16 IntPendingReg;
	UWord16 IntEdgeSensReg;
	UWord16 Reserved[7];
	}; 
	struct{
	 UWord16 pur;
	 UWord16 dr;
	 UWord16 ddr;
	 UWord16 per;
	 UWord16 iar;
	 UWord16 ienr;
	 UWord16 ipolr;
	 UWord16 ipr;
	 UWord16 iesr;
	
	 UWord16 reserved[7];
	}; 
} arch_sPort;

/* peripheral space base addresses */

#define SYS_BASE    0x0C00
#define TMRA_BASE   0x0D00
#define TMRB_BASE	0x0D20
#define TMRC_BASE  	0x0D40
#define TMRD_BASE  	0x0D60
#define CAN_BASE    0x0D80
#define PWMA_BASE  	0x0E00
#define PWMB_BASE  	0x0E20
#define DEC0_BASE   0x0E40
#define DEC1_BASE   0x0E50
#define INTC_BASE  	0x0E60
#define ADCA_BASE  	0x0E80
#define ADCB_BASE  	0x0EC0
#define SCI0_BASE  	0x0F00
#define SCI1_BASE  	0x0F10
#define SPI_BASE  	0x0F20
#define COP_BASE 	0x0F30
#define PFIU_BASE 	0x0F40
#define DFIU_BASE 	0x0F60
#define BFIU_BASE 	0x0F80
#define CLKGEN_BASE	0x0FA0
#define GPIO_A_BASE 0x0FB0
#define GPIO_B_BASE 0x0FC0
#define GPIO_D_BASE 0x0FE0
#define GPIO_E_BASE 0x0FF0

typedef volatile struct{
	arch_sSIM      Sim;
	UWord16        Reserved1[240];
	arch_sTimer    TimerA;
	arch_sTimer    TimerB;
	arch_sTimer    TimerC;
	arch_sTimer    TimerD;
	arch_sMSCAN    MSCan;
	arch_sPWM      PwmA;
	arch_sPWM      PwmB;
	arch_sDecoder  Decoder0;
	arch_sDecoder  Decoder1;
	arch_sIntCntrl IntController;
	arch_sADC      AdcA;
	arch_sADC      AdcB;          /* not available on the DSP56F805 device */
	arch_sSCI      Sci0;
	arch_sSCI      Sci1;
	arch_sSPI      Spi0;
	arch_sCOP      Cop;
	arch_sFlash    ProgramFlash;
	arch_sFlash    DataFlash;
	arch_sFlash    BootFlash;
	arch_sPLL      Pll;
	arch_sPort     PortA;
	arch_sPort     PortB;
	arch_sPort     Reserved;
	arch_sPort     PortD;
	arch_sPort     PortE;
} arch_sIO;

typedef volatile union{
 struct{
	UWord16        Reserved1[0x79];
	UWord16        BusControlReg;
	UWord16        Reserved2;
	UWord16        InterruptPriorityReg;
	UWord16        Reserved3[3];
	UWord16        BusTransferReg;	
 };
 struct{
	UWord16        reserved1[0x79];
	UWord16        bcr;
	UWord16        reserved2;
	UWord16        ipr;
	UWord16        reserved3[3];
	UWord16        opgdbr;	
 };
 
} arch_sCore;

/*
typedef struct{
	UWord32 Channel0;
	UWord32 Channel1;
	UWord32 Channel2;
	UWord32 Channel3;
} arch_sIntTimer;

typedef struct{
	UWord32 TransmitterComplete;
	UWord32 TransmitterReady;
	UWord32 ReceiverError;
	UWord32 ReceiverFull;
} arch_sIntSCI;

typedef struct{
	UWord32        HardwareReset;
	UWord32        COPReset;
	UWord32        Reserved1;
	UWord32        IllegalInstruction;
	UWord32        Software;
	UWord32        HWStackOverflow;
	UWord32        OnCEInstruction;
	UWord32        Reserved2;
	UWord32        IrqA;
	UWord32        IrqB;
	UWord32        Reserved3;
	UWord32        BootFlash;
	UWord32        ProgramFlash;
	UWord32        DataFlash;
	UWord32        MscanTransmiterReady;
	UWord32        MscanReceiverReady;
	UWord32        MscanError;
	UWord32        MscanWakeup;
	UWord32        Reserved4;
	UWord32        MpioE;
	UWord32        MpioD;
	UWord32        Reserved5;
	UWord32        MpioB;
	UWord32        MpioA;
	UWord32        SpiTransmitterEmpty;
	UWord32        SpiReceiverFullError;
	UWord32        Decoder1HomeWatchdog;
	UWord32        Decoder1IndexPulse;
	UWord32        Decoder0HomeWatchdog;
	UWord32        Decoder0IndexPulse;
	arch_sIntTimer TimerD;
	arch_sIntTimer TimerC;
	arch_sIntTimer TimerB;
	arch_sIntTimer TimerA;
	arch_sIntSCI   Sci1;
	arch_sIntSCI   Sci0;
	UWord32        AdcBConversionComplete; 
	UWord32        AdcAConversionComplete;
	UWord32        AdcBZeroCrossLimitError;
	UWord32        AdcAZeroCrossLimitError;
	UWord32        PwmBReload;
	UWord32        PwmAReload;
	UWord32        PwmBFault;
	UWord32        PwmAFault;
	UWord32        PllNoLock;
	UWord32        LowVoltage;
} arch_sInterrupts;
*/

#define ARCH_IO_REGISTERS 0x0C00
//#define ARCH_INTERRUPTS   0x0000

/* The location of the following structures is defined in linker.cmd */
extern arch_sIO           ArchIO;
extern arch_sCore         ArchCore;


/*******************************************************
* Architecture Dependent Routines
*******************************************************/

extern Flag archGetLimitBit (void);

/* void archResetLimitBit (void); */
#define archResetLimitBit() asm(bfclr  #0x40,SR)

/* void archSetNoSat (void); */
#define archSetNoSat() asm(bfclr #0x10,OMR)

/* void archSetSat32 (void); */
#define archSetSat32() asm(bfset #0x10,OMR)

/* Get, then set saturation mode */
extern bool archGetSetSaturationMode (bool bSatMode);

/* void archSet2CompRound (void); */
#define archSet2CompRound() asm(bfset #0x20,OMR)

/* void archSetConvRound (void); */
#define archSetConvRound() asm(bfclr #0x20,OMR)

/* void archStop (void); */
#define archStop() asm(stop)

/* void archTrap (void); */
#define archTrap() asm(swi)

/* void archWait (void); */
#define archWait() asm(wait)

/* void archDebug (void); */
#define archDebug() asm(debug)

/* void archEnableInt (void); */
#define archEnableInt() {asm(bfset #0x0100,SR); asm(bfclr #0x0200,SR);}

/* void archDisableInt (void); */
#define archDisableInt() {asm(bfset #0x0300,SR); asm(nop);}

extern void archDelay(UWord16 Ticks);


/********************************************************
* Interrupt Support - functions for saving and
* restoring all registers
********************************************************/
extern void		archPushAllRegisters (void);
extern void		archPopAllRegisters  (void);


/********************************************************
* Intrinsics and operation primitives workarround
********************************************************/

/* some intrinsic functions are replaced by functions in arch.c */
/* and are implemented as subroutines */
/* when CW will implement better ones define it as in types.h */

/* substitute for shl intrinsic - shift only in 1 direction !!!
   original shl() remain usable, for shifts by immediate number
   the original shl may be better
   Call prototype: Frac16 shl2(Frac16 num,UWord16 shifts);
   macro for parameters exchanching of the shl2p()
*/
#define shl2(par1,par2) shl2p((par2),(par1))
Frac16 shl2p(UWord16 shifts,Frac16 num); /* implementation in arch.c */

/* substitute for shr intrinsic - shift only in 1 direction !!!
   same situation as in shl2()
   Call prototype: Frac16 shr2(Frac16 num,UWord16 shifts);
   macro for parameters exchanching of the shr2p()
*/
#define shr2(par1,par2) shr2p((par2),(par1))
Frac16 shr2p(UWord16 shifts,Frac16 num); /* implementation in arch.c */


/*  Primitives (impyuu, impysu) added to circumvent CW problems */
asm unsigned long impyuu(unsigned short unsigA, unsigned short unsigB);
asm long impysu(short sig, unsigned short unsig);

#endif
