/************************************************************************
 *                                                                      *	
 *  $File Name:     sincos.h$                                           *    
 *  Project:        Sin/Cos Software Driver                             *
 *  Description:    Functions in the module process signals of the      *
 *                  Sin/Cos sensor and returns estimates of the rotor   *
 *                  angular position and speed                          *
 *  Platform:       MC56F8300                                           *
 *  $Version:       1.0.11.0$                                           *         
 *  $Date:          Aug-5-2004$                                         *
 *  Author:         r55013                                              *     
 *  Company:        Freescale SPS Inc.                                  *
 *  Security:       General Business                                    *
 *                                                                      *
 ************************************************************************
 * Copyright (c):   Freescale SPS Inc.,2004,    All rights reserved.    *
 ************************************************************************
 * THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED     *     
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED    *
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   *
 * ARE DISCLAIMED.                                                      *
 * IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY    *
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL   *
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE    *
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS        *
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER *
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR      *
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN  *
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                        *
 ************************************************************************
 */
 #ifndef __SINCOS_H__
    #define __SINCOS_H__ 
    
    #include "types.h"  /* include definitions of the basic data types  */
    
    /********************************************************************
     * exported typedef(s) definitions                                  *
     ********************************************************************/
    typedef Frac16 (*ATANFCN)(Frac16 sin, Frac16 cos);
            
    /********************************************************************
     * exported function prototype(s)                                   *
     ********************************************************************/
    extern Frac32 MCLIB_InitSinCos     (UWord16 nPulses, Frac16 sin, Frac16 cos, 
                                        Frac16 k1_d, Frac16 k2_d, int k1_sc, 
                                        int k2_sc, ATANFCN pFcn);   
    extern Frac32 MCLIB_RawSinCosPos   (Frac32 pulses, Frac16 sin, Frac16 cos);                                             
    extern void   MCLIB_CalcSinCosObs  (Frac32 pulses, Frac16 sin, Frac16 cos);
    extern Frac32 MCLIB_GetSinCosPos   (void);
    extern Frac32 MCLIB_GetSinCosSpeed (void);
    extern int    MCLIB_GetSinCosRev   (void);  
    extern void   MCLIB_SetSinCosPos   (Frac32 pos);
    extern void   MCLIB_SetSinCosRev   (int rev);  
    
#endif /* __SINCOS_H__ */ 
/************************************************************************
 * end of module                                                        * 
 ************************************************************************/